# encoding: utf-8
require "logstash/codecs/base"
require "logstash/timestamp"
require "logstash/util"

class LogStash::Codecs::Transit < LogStash::Codecs::Base
  config_name "transit"

  # Underlying transport to be used. Valid values are 'json' or 'msgpack'
  config :transport, :validate => :string, :default => 'json'
  # Whether messages are base64 encoded or not
  config :base64, :validate => :boolean, :default => false

  public
  def register
    require "transit"
    require "base64" if @base64
  end

  public
  def decode(data)
    begin
      data = Base64.decode64(data) if @base64
      io = StringIO.new data
      reader = Transit::Reader.new(@transport.to_sym, io)
      event = LogStash::Event.new(reader.read)
      event["tags"] ||= []
    rescue => e
      @logger.warn("Trouble parsing transit input, falling back to plain text",
                   :input => data, :exception => e)
      event = LogStash::Event.new("") unless event
      event["message"] = data
      event["tags"] ||= []
      event["tags"] << "_transitparsefailure"
    end
    yield event
  end

  public
  def encode(event)
    data = LogStash::Util.normalize(event.to_hash)
    io = StringIO.new
    writer = Transit::Writer.new(@transport.to_sym, io)
    writer.write(data.merge(LogStash::Event::TIMESTAMP => event.timestamp.to_iso8601))

    encoded = io.string
    encoded = Base64.encode64(encoded) if @base64
    @on_event.call(event, encoded)
  end

end
